@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2007-2023, OFFIS e.V.
+ *  Copyright (C) 2007-2025, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -171,7 +171,7 @@ static void addCmdLineOptions(OFCommandLine& cmd)
       cmd.addOption("--transliterate",       "-Ct",     "try to approximate characters that cannot be\nrepresented through similar looking characters");
       cmd.addOption("--discard-illegal",     "-Cd",     "discard characters that cannot be represented\nin destination character set");
     cmd.addSubGroup("other processing options:");
-      cmd.addOption("--key",                 "-k",   1, "[k]ey: gggg,eeee=\"str\", path or dict. name=\"str\"",
+      cmd.addOption("--key",                 "-k",   1, "[k]ey: gggg,eeee=\"str\", path or dict name=\"str\"",
                                                         "add further attribute");
 
   cmd.addGroup("output options:");
@@ -469,6 +469,9 @@ static OFCondition startConversion(
     cond = i2d.convertNextFrame(inputPlug, ++frameNum);
   }
 
+  // adjust byte order of pixel data to local OW byte order
+  if (cond.good()) cond = i2d.adjustByteOrder(inputFiles.size());
+
   // update offset table if image type is encapsulated
   if (cond.good()) cond = i2d.updateOffsetTable();
 
