@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2002-2020, OFFIS e.V.
+ *  Copyright (C) 2002-2025, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -21,6 +21,7 @@
 
 #include "dcmtk/config/osconfig.h"
 #include "dcmtk/dcmdata/dcrleccd.h"
+#include "dcmtk/ofstd/ofstd.h"
 
 // dcmdata includes
 #include "dcmtk/dcmdata/dcrlecp.h"   /* for class DcmRLECodecParameter */
@@ -51,13 +52,28 @@ OFBool DcmRLECodecDecoder::canChangeCoding(
 {
   E_TransferSyntax myXfer = EXS_RLELossless;
   DcmXfer newRep(newRepType);
-  if (newRep.isNotEncapsulated() && (oldRepType == myXfer)) return OFTrue; // decompress requested
+  if (newRep.usesNativeFormat() && (oldRepType == myXfer))
+    return OFTrue; // decompress requested
 
   // we don't support re-coding for now.
   return OFFalse;
 }
 
 
+Uint16 DcmRLECodecDecoder::decodedBitsAllocated(
+    Uint16 bitsAllocated,
+    Uint16 /* bitsStored */) const
+{
+    // The RLE decoder only supports images where BitsAllocated is a multiple of 8.
+    if ((bitsAllocated < 8)||(bitsAllocated % 8 != 0)) return 0;
+
+    // RLE cannot support more than 120 bits/sample (15 bands) in DICOM
+    if (bitsAllocated > 120) return 0;
+
+    return bitsAllocated;
+}
+
+
 OFCondition DcmRLECodecDecoder::decode(
     const DcmRepresentationParameter * /* fromRepParam */,
     DcmPixelSequence * pixSeq,
@@ -129,9 +145,25 @@ OFCondition DcmRLECodecDecoder::decode(
       if (rledecoder.fail()) result = EC_MemoryExhausted;  // RLE decoder failed to initialize
       else
       {
-        const size_t frameSize = OFstatic_cast(size_t, imageBytesAllocated) * OFstatic_cast(size_t, imageRows)
-            * OFstatic_cast(size_t, imageColumns) * OFstatic_cast(size_t, imageSamplesPerPixel);
-        size_t totalSize = frameSize * imageFrames;
+        // compute size of uncompressed frame, in bytes
+        Uint32 frameSize = imageBytesAllocated * imageRows * imageColumns * imageSamplesPerPixel;
+
+        // check for overflow
+        if (imageRows != 0 && frameSize / imageRows != OFstatic_cast(Uint32, imageBytesAllocated) * imageColumns * imageSamplesPerPixel)
+        {
+          DCMDATA_WARN("DcmRLECodecDecoder: Cannot decompress image because uncompressed representation would exceed maximum possible size of PixelData attribute");
+          return EC_ElemLengthExceeds32BitField;
+        }
+
+        Uint32 totalSize = frameSize * imageFrames;
+
+        // check for overflow
+        if (totalSize == 0xFFFFFFFF || (frameSize != 0 && totalSize / frameSize != OFstatic_cast(Uint32, imageFrames)))
+        {
+          DCMDATA_WARN("DcmRLECodecDecoder: Cannot decompress image because uncompressed representation would exceed maximum possible size of PixelData attribute");
+          return EC_ElemLengthExceeds32BitField;
+        }
+
         if (totalSize & 1) totalSize++; // align on 16-bit word boundary
         Uint16 *imageData16 = NULL;
         Sint32 currentFrame = 0;
@@ -139,7 +171,7 @@ OFCondition DcmRLECodecDecoder::decode(
         Uint32 numberOfStripes = 0;
         Uint32 fragmentLength = 0;
 
-        result = uncompressedPixelData.createUint16Array(OFstatic_cast(Uint32, totalSize/sizeof(Uint16)), imageData16);
+        result = uncompressedPixelData.createUint16Array(totalSize/sizeof(Uint16), imageData16);
         if (result.good())
         {
           Uint8 *imageData8 = OFreinterpret_cast(Uint8 *, imageData16);
@@ -280,7 +312,7 @@ OFCondition DcmRLECodecDecoder::decode(
                     {
                       if (rledecoder.size() < bytesPerStripe)
                       {
-                        DCMDATA_WARN("RLE decoder is finished but has produced insufficient data for this stripe, will continue with next pixel item");
+                        DCMDATA_WARN("DcmRLECodecDecoder: Detected fragmented RLE compressed pixel data, which is not allowed in DICOM");
                         DCMDATA_DEBUG("RLE decoder processes pixel item " << currentItem);
                         result = pixSeq->getItem(pixItem, currentItem++);
                         if (result.good())
@@ -316,6 +348,7 @@ OFCondition DcmRLECodecDecoder::decode(
 
                       if (result.good() || result == EC_StreamNotifyClient)
                       {
+                        DCMDATA_WARN("DcmRLECodecDecoder: Detected fragmented RLE compressed pixel data, which is not allowed in DICOM");
                         DCMDATA_DEBUG("RLE decoder processes pixel item " << currentItem);
                         result = pixSeq->getItem(pixItem, currentItem++);
                       }
@@ -330,6 +363,12 @@ OFCondition DcmRLECodecDecoder::decode(
                     } /* while */
 
                     // last fragment for this RLE stripe
+                    if (inputBytes + byteOffset > fragmentLength)
+                    {
+                        DCMDATA_WARN("DcmRLECodecDecoder: Stream size in RLE header is wrong, adjusting from " << inputBytes << " to " << fragmentLength-byteOffset << " bytes");
+                        inputBytes = fragmentLength-byteOffset;
+                    }
+
                     result = rledecoder.decompress(rleData + byteOffset, OFstatic_cast(size_t, inputBytes));
 
                     // special handling for zero pad byte at the end of the RLE stream
@@ -348,7 +387,7 @@ OFCondition DcmRLECodecDecoder::decode(
                 if (lastStripeOfColor && (rledecoder.size() < bytesPerStripe))
                 {
                     // stripe ended prematurely? report a warning and continue
-                    DCMDATA_WARN("RLE decoder is finished but has produced insufficient data for this stripe, filling remaining pixels");
+                    DCMDATA_WARN("DcmRLECodecDecoder: RLE decoder is finished but has produced insufficient data for this stripe, filling remaining pixels");
                     result = EC_Normal;
                 }
                 else if (rledecoder.size() != bytesPerStripe)
@@ -416,7 +455,7 @@ OFCondition DcmRLECodecDecoder::decode(
           if (result.good() && (numberOfFramesPresent || (imageFrames > 1)))
           {
             char numBuf[20];
-            sprintf(numBuf, "%ld", OFstatic_cast(long, imageFrames));
+            OFStandard::snprintf(numBuf, sizeof(numBuf), "%ld", OFstatic_cast(long, imageFrames));
             result = OFstatic_cast(DcmItem *, dataset)->putAndInsertString(DCM_NumberOfFrames, numBuf);
           }
         }
@@ -463,7 +502,7 @@ OFCondition DcmRLECodecDecoder::decodeFrame(
     Uint16 imageColumns = 0;
     Sint32 imageFrames = 1;
     Uint16 imageBitsAllocated = 0;
-    Uint16 imageBytesAllocated = 0;
+    Uint32 imageBytesAllocated = 0;
     Uint16 imagePlanarConfiguration = 0;
     Uint32 rleHeader[16];
     OFString photometricInterpretation;
@@ -476,7 +515,7 @@ OFCondition DcmRLECodecDecoder::decodeFrame(
     if (result.good()) result = dataset->findAndGetOFString(DCM_PhotometricInterpretation, photometricInterpretation);
     if (result.good())
     {
-        imageBytesAllocated = OFstatic_cast(Uint16, imageBitsAllocated / 8);
+        imageBytesAllocated = OFstatic_cast(Uint32, imageBitsAllocated / 8);
         if ((imageBitsAllocated < 8)||(imageBitsAllocated % 8 != 0))
         {
           DCMDATA_ERROR("The RLE decoder only supports images where BitsAllocated is a multiple of 8.");
@@ -500,9 +539,16 @@ OFCondition DcmRLECodecDecoder::decodeFrame(
     const size_t bytesPerStripe = OFstatic_cast(size_t, imageColumns) * OFstatic_cast(size_t, imageRows);
     Uint32 numberOfStripes = 0;
     Uint32 fragmentLength = 0;
-    Uint32 frameSize = OFstatic_cast(Uint32, imageBytesAllocated) * OFstatic_cast(Uint32, imageRows)
+    Uint32 frameSize = imageBytesAllocated * OFstatic_cast(Uint32, imageRows)
                        * OFstatic_cast(Uint32, imageColumns) * OFstatic_cast(Uint32, imageSamplesPerPixel);
 
+    // check for overflow
+    if (imageRows != 0 && frameSize / imageRows != (imageBytesAllocated * imageColumns * imageSamplesPerPixel))
+    {
+      DCMDATA_WARN("DcmRLECodecDecoder: Cannot decompress image because uncompressed representation would exceed maximum possible size of PixelData attribute");
+      return EC_ElemLengthExceeds32BitField;
+    }
+
     if (frameSize > bufSize) return EC_IllegalCall;
 
     DcmRLEDecoder rledecoder(bytesPerStripe);
@@ -592,6 +638,11 @@ OFCondition DcmRLECodecDecoder::decodeFrame(
             // if the RLE data is split in multiple fragments. We need to feed
             // data fragment by fragment until the RLE codec has produced
             // sufficient output.
+            if (fragmentLength < byteOffset)
+            {
+              DCMDATA_ERROR("Byte offset in RLE header is wrong.");
+              return EC_CannotChangeRepresentation;
+            }
             bytesToDecode = OFstatic_cast(size_t, fragmentLength - byteOffset);
         }
         else
@@ -610,7 +661,13 @@ OFCondition DcmRLECodecDecoder::decodeFrame(
             bytesToDecode = OFstatic_cast(size_t, inputBytes);
         }
 
-        // last fragment for this RLE stripe
+        // make sure we don't overshoot the buffer size in case of an incorrect byte offset
+        if (fragmentLength < byteOffset + bytesToDecode)
+        {
+          DCMDATA_ERROR("Byte offset in RLE header is wrong.");
+          return EC_CannotChangeRepresentation;
+        }
+
         result = rledecoder.decompress(rleData + byteOffset, bytesToDecode);
 
         // special handling for zero pad byte at the end of the RLE stream
@@ -693,6 +750,10 @@ OFCondition DcmRLECodecDecoder::decodeFrame(
     }
 
     // adjust byte order for uncompressed image to little endian
+    if ((gLocalByteOrder == EBO_BigEndian) && (frameSize & 1))
+    {
+      DCMDATA_WARN("Size of frame buffer is odd, cannot correct byte order for last pixel value");
+    }
     swapIfNecessary(EBO_LittleEndian, gLocalByteOrder, imageData16, frameSize, sizeof(Uint16));
 
     return result;
