@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2002-2020, OFFIS e.V.
+ *  Copyright (C) 2002-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -31,10 +31,6 @@
 #include "dcmtk/dcmdata/dcitem.h"
 #include "dcmtk/ofstd/ofstd.h"
 
-#define INCLUDE_CSTDIO
-#include "dcmtk/ofstd/ofstdinc.h"
-
-
 typedef OFList<DcmRLEEncoder *> DcmRLEEncoderList;
 typedef OFListIterator(DcmRLEEncoder *) DcmRLEEncoderListIterator;
 
@@ -59,13 +55,21 @@ OFBool DcmRLECodecEncoder::canChangeCoding(
   E_TransferSyntax myXfer = EXS_RLELossless;
   DcmXfer newRep(newRepType);
   DcmXfer oldRep(oldRepType);
-  if (oldRep.isNotEncapsulated() && (newRepType == myXfer)) return OFTrue; // compress requested
+  if (oldRep.usesNativeFormat() && (newRepType == myXfer)) return OFTrue; // compress requested
 
   // we don't support re-coding for now.
   return OFFalse;
 }
 
 
+Uint16 DcmRLECodecEncoder::decodedBitsAllocated(
+    Uint16 /* bitsAllocated */,
+    Uint16 /* bitsStored */) const
+{
+  return 0;
+}
+
+
 OFCondition DcmRLECodecEncoder::decode(
     const DcmRepresentationParameter * /* fromRepParam */,
     DcmPixelSequence * /* pixSeq */,
