@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2015-2022, Open Connections GmbH
+ *  Copyright (C) 2015-2025, Open Connections GmbH
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation are maintained by
@@ -19,14 +19,15 @@
  *
  */
 
-#include "dcmtk/dcmiod/modmultiframedimension.h"
 #include "dcmtk/config/osconfig.h"
+#include "dcmtk/dcmiod/modmultiframedimension.h"
 #include "dcmtk/dcmdata/dcdeftag.h"
 #include "dcmtk/dcmdata/dcpath.h" // for private tag reservation checking
 #include "dcmtk/dcmdata/dcvrat.h"
 #include "dcmtk/dcmdata/dcvrcs.h"
 #include "dcmtk/dcmdata/dcvrlo.h"
 #include "dcmtk/dcmdata/dcvrui.h"
+#include "dcmtk/dcmdata/dcdeftag.h"
 #include "dcmtk/dcmiod/iodutil.h" // for static helpers
 
 const OFString IODMultiframeDimensionModule::m_ModuleName = "MultiframeDimensionModule";
@@ -45,6 +46,7 @@ IODMultiframeDimensionModule::IODMultiframeDimensionModule()
     : IODModule()
     , m_DimensionIndexSequence()
     , m_DimensionOrganizationSequence()
+    , m_CheckOnWrite(OFTrue)
 {
     resetRules();
 }
@@ -199,8 +201,8 @@ OFCondition IODMultiframeDimensionModule::checkDimensions(DcmItem* fgItem)
     DcmSequenceOfItems* perFrame = NULL;
     if (fgItem->findAndGetSequence(DCM_PerFrameFunctionalGroupsSequence, perFrame).bad())
     {
-        DCMIOD_WARN(
-            "Will not check dimension consistency with functional groups (no per-frame functional groups found)");
+        DCMIOD_DEBUG(
+            "Omitting dimension consistency check with functional groups (Per-frame FGs not yet present)");
     }
 
     OFCondition result;
@@ -397,7 +399,7 @@ void IODMultiframeDimensionModule::createDimensionOrganizationData()
             DimensionOrganizationItem* uidItem = new DimensionOrganizationItem();
             if (!uidItem)
             {
-                DCMIOD_ERROR("Memory Exhausted while collecting Dimension Organziation UIDs");
+                DCMIOD_ERROR("Memory Exhausted while collecting Dimension Organization UIDs");
                 return;
             }
             result = uidItem->setDimensionOrganizationUID(newuid);
