@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2007-2020, OFFIS e.V.
+ *  Copyright (C) 2007-2025, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -29,9 +29,6 @@
 #include "dcmtk/ofstd/offile.h"      /* for class OFFile */
 #include "dcmtk/ofstd/ofbmanip.h"
 
-#define INCLUDE_CMATH
-#include "dcmtk/ofstd/ofstdinc.h"
-
 // dcmdata includes
 #include "dcmtk/dcmdata/dcdatset.h"  /* for class DcmDataset */
 #include "dcmtk/dcmdata/dcdeftag.h"  /* for tag constants */
@@ -58,15 +55,9 @@
 #include "intrface.h"
 
 BEGIN_EXTERN_C
-#ifdef HAVE_FCNTL_H
 #include <fcntl.h>       /* for O_RDONLY */
-#endif
-#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>   /* required for sys/stat.h */
-#endif
-#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>    /* for stat, fstat */
-#endif
 END_EXTERN_C
 
 
@@ -99,7 +90,15 @@ OFBool DJLSEncoderBase::canChangeCoding(
 {
   // this codec only handles conversion from uncompressed to JPEG-LS.
   DcmXfer oldRep(oldRepType);
-  return (oldRep.isNotEncapsulated() && (newRepType == supportedTransferSyntax()));
+  return (oldRep.usesNativeFormat() && (newRepType == supportedTransferSyntax()));
+}
+
+
+Uint16 DJLSEncoderBase::decodedBitsAllocated(
+    Uint16 /* bitsAllocated */,
+    Uint16 /* bitsStored */) const
+{
+  return 0;
 }
 
 
@@ -375,7 +374,7 @@ OFCondition DJLSEncoderBase::updateDerivationDescription(
   derivationDescription =  "near lossless JPEG-LS compression, factor ";
   OFStandard::ftoa(buf, sizeof(buf), ratio, OFStandard::ftoa_uppercase, 0, 5);
   derivationDescription += buf;
-  sprintf(buf, " (NEAR=%lu)", OFstatic_cast(unsigned long, djrp->getnearlosslessDeviation()));
+  OFStandard::snprintf(buf, sizeof(buf), " (NEAR=%lu)", OFstatic_cast(unsigned long, djrp->getnearlosslessDeviation()));
   derivationDescription += buf;
 
   // append old Derivation Description, if any
@@ -465,10 +464,23 @@ OFCondition DJLSEncoderBase::losslessRawEncode(
       // an image that is not supported by either the raw or the cooked encoder.
       result = EC_JLSUnsupportedImageType;
     }
+  }
 
+  if (result.good())
+  {
     // make sure that all the descriptive attributes have sensible values
     if ((columns < 1)||(rows < 1)||(samplesPerPixel < 1)) result = EC_JLSUnsupportedImageType;
+  }
 
+  if (result.good())
+  {
+    // we do not support JPEG-LS compression of YBR_FULL_422 images
+    if (photometricInterpretation == "YBR_FULL_422")
+      result = EC_JLSUnsupportedImageType;
+  }
+
+  if (result.good())
+  {
     // make sure that we have at least as many bytes of pixel data as we expect
     if (bytesAllocated * samplesPerPixel * columns * rows *
       OFstatic_cast(unsigned long,numberOfFrames) > length)
@@ -656,7 +668,7 @@ OFCondition DJLSEncoderBase::compressRawFrame(
   jls_params.width = width;
   jls_params.allowedlossyerror = 0; // must be zero for raw mode
   jls_params.outputBgr = false;
-  // No idea what this one does, but I don't think DICOM says anything about it
+  // color transformation is a non-standard HP/JPEG-LS extension
   jls_params.colorTransform = 0;
   // Unset: jls_params.jfif (thumbnail, dpi)
 
@@ -722,7 +734,7 @@ OFCondition DJLSEncoderBase::compressRawFrame(
   if ((jls_params.ilv == ILV_NONE && (ilv == ILV_SAMPLE || ilv == ILV_LINE)) ||
       (ilv == ILV_NONE && (jls_params.ilv == ILV_SAMPLE || jls_params.ilv == ILV_LINE)))
   {
-    DCMJPLS_DEBUG("Converting image from " << (ilv == ILV_NONE ? "color-by-plane" : "color-by-pixel")
+    DCMJPLS_DEBUG("converting image from " << (ilv == ILV_NONE ? "color-by-plane" : "color-by-pixel")
           << " to " << (jls_params.ilv == ILV_NONE ? "color-by-plane" : "color-by-pixel"));
 
     frameBuffer = new Uint8[frameSize];
@@ -784,7 +796,7 @@ OFCondition DJLSEncoderBase::losslessCookedEncode(
   if (result.good()) result = dataset->findAndGetUint16(DCM_BitsAllocated, bitsAllocated);
   if (result.bad()) return result;
 
-  // The cooked encoder only handles the following photometic interpretations
+  // The cooked encoder only handles the following photometric interpretations
   if (photometricInterpretation != "MONOCHROME1" &&
       photometricInterpretation != "MONOCHROME2" &&
       photometricInterpretation != "RGB" &&
@@ -822,12 +834,12 @@ OFCondition DJLSEncoderBase::losslessCookedEncode(
 
   if (bitsStored > 16)
   {
-    DCMJPLS_WARN("Cannot compress image with " << bitsStored << " bits/sample: JPEG-LS supports max. 16 bits.");
+    DCMJPLS_WARN("cannot compress image with " << bitsStored << " bits/sample: JPEG-LS supports max. 16 bits");
     return EC_JLSUnsupportedBitDepth;
   }
   if (bitsStored < 2)
   {
-    DCMJPLS_WARN("Cannot compress image with " << bitsStored << " bit/sample: JPEG-LS requires at least 2 bits.");
+    DCMJPLS_WARN("cannot compress image with " << bitsStored << " bit/sample: JPEG-LS requires at least 2 bits");
     return EC_JLSUnsupportedBitDepth;
   }
 
@@ -920,8 +932,8 @@ OFCondition DJLSEncoderBase::losslessCookedEncode(
         result = dataset->putAndInsertUint16(DCM_BitsAllocated, 16);
       else
         result = dataset->putAndInsertUint16(DCM_BitsAllocated, 8);
-    if (result.good()) result = dataset->putAndInsertUint16(DCM_BitsStored, bitsPerSample);
-    if (result.good()) result = dataset->putAndInsertUint16(DCM_HighBit, bitsPerSample-1);
+    if (result.good()) result = dataset->putAndInsertUint16(DCM_BitsStored, OFstatic_cast(Uint16, bitsPerSample));
+    if (result.good()) result = dataset->putAndInsertUint16(DCM_HighBit, OFstatic_cast(Uint16, (bitsPerSample-1)));
     if (result.good())
     {
       if (photometricInterpretation == "RGB" || photometricInterpretation == "YBR_FULL")
@@ -1005,12 +1017,17 @@ OFCondition DJLSEncoderBase::compressCookedFrame(
     case EPR_Sint8:
       {
         // image representation is 8 bit signed or unsigned
+        Uint8 mask = 0xFF >> (8-depth);
         if (samplesPerPixel == 1)
         {
           const Uint8 *yv = OFreinterpret_cast(const Uint8 *, planes[0]) + framesize * frame;
           buffer_size = framesize;
           buffer = new Uint8[buffer_size];
-          memcpy(buffer, yv, framesize);
+          for (size_t i=0; i < framesize; ++i)
+          {
+              buffer[i] = *yv & mask;
+              yv++;
+          }
         }
         else
         {
@@ -1026,9 +1043,9 @@ OFCondition DJLSEncoderBase::compressCookedFrame(
           {
             for (int col=width; col; --col)
             {
-              buffer[i++] = *rv;
-              buffer[i++] = *gv;
-              buffer[i++] = *bv;
+              buffer[i++] = *rv & mask;
+              buffer[i++] = *gv & mask;
+              buffer[i++] = *bv & mask;
 
               rv++;
               gv++;
@@ -1042,12 +1059,23 @@ OFCondition DJLSEncoderBase::compressCookedFrame(
     case EPR_Sint16:
       {
         // image representation is 16 bit signed or unsigned
+        Uint16 mask = 0xFFFF >> (16-depth);
         if (samplesPerPixel == 1)
         {
           const Uint16 *yv = OFreinterpret_cast(const Uint16 *, planes[0]) + framesize * frame;
-          buffer_size = framesize*sizeof(Uint16);
-          buffer = new Uint8[buffer_size];
-          memcpy(buffer, yv, buffer_size);
+
+          buffer_size = framesize;
+          Uint16 *buffer16 = new Uint16[buffer_size];
+          buffer = OFreinterpret_cast(Uint8 *, buffer16);
+
+          // Convert to byte count
+          buffer_size *= 2;
+
+          for (size_t i=0; i < framesize; ++i)
+          {
+              buffer16[i] = *yv & mask;
+              yv++;
+          }
         }
         else
         {
@@ -1067,9 +1095,9 @@ OFCondition DJLSEncoderBase::compressCookedFrame(
           {
             for (int col=width; col; --col)
             {
-              buffer16[i++] = *rv;
-              buffer16[i++] = *gv;
-              buffer16[i++] = *bv;
+              buffer16[i++] = *rv & mask;
+              buffer16[i++] = *gv & mask;
+              buffer16[i++] = *bv & mask;
 
               rv++;
               gv++;
@@ -1094,7 +1122,7 @@ OFCondition DJLSEncoderBase::compressCookedFrame(
   jls_params.allowedlossyerror = nearLosslessDeviation;
   jls_params.outputBgr = false;
   jls_params.bitspersample = depth;
-  // No idea what this one does, but I don't think DICOM says anything about it
+  // color transformation is a non-standard HP/JPEG-LS extension
   jls_params.colorTransform = 0;
 
   // This was already checked for a sane value above
@@ -1103,7 +1131,7 @@ OFCondition DJLSEncoderBase::compressCookedFrame(
   // Unset: jls_params.jfif (thumbnail, dpi)
 
   // set parameters T1, T2, T3, MAXVAL and RESET
-  setCustomParameters(jls_params.custom, depth, nearLosslessDeviation, djcp);
+  setCustomParameters(jls_params.custom, OFstatic_cast(Uint16, depth), nearLosslessDeviation, djcp);
 
   switch (djcp->getJplsInterleaveMode())
   {
@@ -1138,11 +1166,11 @@ OFCondition DJLSEncoderBase::compressCookedFrame(
   // Do we have to convert the image to color-by-plane now?
   if (jls_params.ilv == ILV_NONE && jls_params.components != 1)
   {
-    DCMJPLS_DEBUG("Converting image from color-by-pixel to color-by-plane");
+    DCMJPLS_DEBUG("converting image from color-by-pixel to color-by-plane");
 
     frameBuffer = new Uint8[buffer_size];
     framePointer = frameBuffer;
-    result = convertToUninterleaved(frameBuffer, buffer, samplesPerPixel, width, height, jls_params.bitspersample);
+    result = convertToUninterleaved(frameBuffer, buffer, OFstatic_cast(Uint16, samplesPerPixel), width, height, OFstatic_cast(Uint16, jls_params.bitspersample));
   }
 #endif
 
@@ -1158,7 +1186,7 @@ OFCondition DJLSEncoderBase::compressCookedFrame(
   {
     // 'compressed_buffer_size' now contains the size of the compressed data in buffer
     compressedSize = OFstatic_cast(unsigned long, bytesWritten);
-    fixPaddingIfNecessary(OFstatic_cast(Uint8 *, buffer), compressed_buffer_size, compressedSize, djcp->getUseFFbitstreamPadding());
+    fixPaddingIfNecessary(OFstatic_cast(Uint8 *, compressed_buffer), compressed_buffer_size, compressedSize, djcp->getUseFFbitstreamPadding());
     result = pixelSequence->storeCompressedFrame(offsetList, compressed_buffer, compressedSize, fragmentSize);
   }
 
@@ -1178,7 +1206,7 @@ OFCondition DJLSEncoderBase::convertToUninterleaved(
     Uint32 height,
     Uint16 bitsAllocated) const
 {
-  Uint8 bytesAllocated = bitsAllocated / 8;
+  Uint8 bytesAllocated = OFstatic_cast(Uint8, (bitsAllocated / 8));
   Uint32 planeSize = width * height * bytesAllocated;
 
   if (bitsAllocated % 8 != 0)
@@ -1203,7 +1231,7 @@ OFCondition DJLSEncoderBase::convertToSampleInterleaved(
     Uint32 height,
     Uint16 bitsAllocated) const
 {
-  Uint8 bytesAllocated = bitsAllocated / 8;
+  Uint8 bytesAllocated = OFstatic_cast(Uint8, (bitsAllocated / 8));
   Uint32 planeSize = width * height * bytesAllocated;
 
   if (bitsAllocated % 8 != 0)
