@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1994-2019, OFFIS e.V.
+ *  Copyright (C) 1994-2025, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were partly developed by
@@ -78,30 +78,8 @@
 
 #include "dcmtk/config/osconfig.h"    /* make sure OS specific configuration is included first */
 
-#define INCLUDE_CSTDLIB
-#define INCLUDE_CSTDIO
-#define INCLUDE_CSTRING
-#define INCLUDE_CCTYPE
-#include "dcmtk/ofstd/ofstdinc.h"
-
-#ifdef HAVE_UNIX_H
-#if defined(macintosh) && defined (HAVE_WINSOCK_H)
-/* unix.h defines timeval incompatible with winsock.h */
-#define timeval _UNWANTED_timeval
-#endif
-#include <unix.h>       /* for unlink() under Metrowerks C++ (Macintosh) */
-#undef timeval
-#endif
-#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
-#endif
-#ifdef HAVE_STAT_H
-#include <stat.h>
-#endif
-
 #include "dcmtk/ofstd/ofstd.h"
 #include "dcmtk/dcmnet/diutil.h"
 #include "dcmtk/dcmdata/dcdatset.h"
@@ -124,7 +102,7 @@ DU_stripTrailingSpaces(char *s)
     if (s)
     {
         n = OFstatic_cast(int, strlen(s));
-        for (i = n - 1; i >= 0 && isspace(TO_UCHAR(s[i])); i--)
+        for (i = n - 1; i >= 0 && OFStandard::isspace(s[i]); i--)
             s[i] = '\0';
     }
 }
@@ -137,10 +115,10 @@ DU_stripLeadingSpaces(char *s)
     if (s == NULL) return;
     n = OFstatic_cast(int, strlen(s));
     if (n == 0) return;
-    if (!isspace(TO_UCHAR(s[0]))) return; /* no leading space */
+    if (!OFStandard::isspace(s[0])) return; /* no leading space */
 
     /* first non-space */
-    for (i=0; i<n && isspace(TO_UCHAR(s[i])); i++)
+    for (i=0; i<n && OFStandard::isspace(s[i]); i++)
         /* do nothing, just iterate */
         ;
     if (i<n) {
@@ -165,22 +143,21 @@ DU_stripLeadingAndTrailingSpaces(char *s)
 OFBool
 DU_getStringDOElement(DcmItem *obj, DcmTagKey t, char *s, size_t bufsize)
 {
-    DcmByteString *elem;
     DcmStack stack;
-    OFCondition ec = EC_Normal;
     char* aString;
 
-    ec = obj->search(t, stack);
-    elem = (DcmByteString*) stack.top();
-    if (ec == EC_Normal && elem != NULL) {
+    OFCondition ec = obj->search(t, stack);
+    if (ec.good() && (stack.top() != NULL) && stack.top()->isElement()) {
+        DcmElement *elem = (DcmElement *) stack.top();
         if (elem->getLength() == 0) {
             s[0] = '\0';
         } else {
             ec =  elem->getString(aString);
-            OFStandard::strlcpy(s, aString, bufsize);
+            if (ec.good())
+                OFStandard::strlcpy(s, aString, bufsize);
         }
     }
-    return (ec == EC_Normal);
+    return (ec.good());
 }
 
 OFBool
@@ -198,7 +175,7 @@ DU_putStringDOElement(DcmItem *obj, DcmTagKey t, const char *s)
         ec = obj->insert(e, OFTrue);
     }
 
-    return (ec == EC_Normal);
+    return (ec.good());
 }
 
 OFBool
@@ -206,15 +183,15 @@ DU_getShortDOElement(DcmItem *obj, DcmTagKey t, Uint16 *us)
 {
     DcmElement *elem;
     DcmStack stack;
-    OFCondition ec = EC_Normal;
 
-    ec = obj->search(t, stack);
-    elem = (DcmElement*) stack.top();
-    if (ec == EC_Normal && elem != NULL) {
-        ec = elem->getUint16(*us, 0);
+    OFCondition ec = obj->search(t, stack);
+    if (ec.good() && stack.top()->isElement())
+    {
+        elem = (DcmElement*) stack.top();
+        if (elem) ec = elem->getUint16(*us, 0);
     }
 
-    return (ec == EC_Normal);
+    return (ec.good());
 }
 
 OFBool
@@ -231,7 +208,7 @@ DU_putShortDOElement(DcmItem *obj, DcmTagKey t, Uint16 us)
     if (ec == EC_Normal) {
         ec = obj->insert(e, OFTrue);
     }
-    return (ec == EC_Normal);
+    return (ec.good());
 }
 
 OFBool
@@ -292,7 +269,7 @@ DU_cechoStatusString(Uint16 statusCode)
     if (statusCode == STATUS_Success)
         s = "Success";
     else {
-        sprintf(staticBuf, "Unknown Status: 0x%x", (unsigned int)statusCode);
+        OFStandard::snprintf(staticBuf, sizeof(staticBuf), "Unknown Status: 0x%x", (unsigned int)statusCode);
         s = staticBuf;
     }
     return s;
@@ -341,7 +318,7 @@ DU_cstoreStatusString(Uint16 statusCode)
     }
 
     if (s == NULL) {
-        sprintf(staticBuf, "Unknown Status: 0x%x", (unsigned int)statusCode);
+        OFStandard::snprintf(staticBuf, sizeof(staticBuf), "Unknown Status: 0x%x", (unsigned int)statusCode);
         s = staticBuf;
     }
     return s;
@@ -366,8 +343,8 @@ DU_cfindStatusString(Uint16 statusCode)
       case STATUS_FIND_Refused_SOPClassNotSupported:
           s = "Refused: SOPClassNotSupported";
           break;
-      case STATUS_FIND_Failed_IdentifierDoesNotMatchSOPClass:
-          s = "Failed: IdentifierDoesNotMatchSOPClass";
+      case STATUS_FIND_Error_DataSetDoesNotMatchSOPClass:
+          s = "Error: DataSetDoesNotMatchSOPClass";
           break;
       case STATUS_FIND_Cancel_MatchingTerminatedDueToCancelRequest:
           s = "Cancel: MatchingTerminatedDueToCancelRequest";
@@ -386,7 +363,7 @@ DU_cfindStatusString(Uint16 statusCode)
     }
 
     if (s == NULL) {
-        sprintf(staticBuf, "Unknown Status: 0x%x", (unsigned int)statusCode);
+        OFStandard::snprintf(staticBuf, sizeof(staticBuf), "Unknown Status: 0x%x", (unsigned int)statusCode);
         s = staticBuf;
     }
     return s;
@@ -410,14 +387,14 @@ DU_cmoveStatusString(Uint16 statusCode)
       case STATUS_MOVE_Refused_OutOfResourcesSubOperations:
           s = "Refused: OutOfResourcesSubOperations";
           break;
-      case STATUS_MOVE_Failed_SOPClassNotSupported:
-          s = "Failed: SOPClassNotSupported";
+      case STATUS_MOVE_Refused_SOPClassNotSupported:
+          s = "Refused: SOPClassNotSupported";
           break;
-      case STATUS_MOVE_Failed_MoveDestinationUnknown:
-          s = "Failed: MoveDestinationUnknown";
+      case STATUS_MOVE_Refused_MoveDestinationUnknown:
+          s = "Refused: MoveDestinationUnknown";
           break;
-      case STATUS_MOVE_Failed_IdentifierDoesNotMatchSOPClass:
-          s = "Failed: IdentifierDoesNotMatchSOPClass";
+      case STATUS_MOVE_Error_DataSetDoesNotMatchSOPClass:
+          s = "Error: DataSetDoesNotMatchSOPClass";
           break;
       case STATUS_MOVE_Cancel_SubOperationsTerminatedDueToCancelIndication:
           s = "Cancel: SubOperationsTerminatedDueToCancelIndication";
@@ -437,7 +414,7 @@ DU_cmoveStatusString(Uint16 statusCode)
     }
 
     if (s == NULL) {
-        sprintf(staticBuf, "Unknown Status: 0x%x", (unsigned int)statusCode);
+        OFStandard::snprintf(staticBuf, sizeof(staticBuf), "Unknown Status: 0x%x", (unsigned int)statusCode);
         s = staticBuf;
     }
     return s;
@@ -461,11 +438,11 @@ DU_cgetStatusString(Uint16 statusCode)
       case STATUS_GET_Refused_OutOfResourcesSubOperations:
           s = "Refused: OutOfResourcesSubOperations";
           break;
-      case STATUS_GET_Failed_SOPClassNotSupported:
-          s = "Failed: SOPClassNotSupported";
+      case STATUS_GET_Refused_SOPClassNotSupported:
+          s = "Refused: SOPClassNotSupported";
           break;
-      case STATUS_GET_Failed_IdentifierDoesNotMatchSOPClass:
-          s = "Failed: IdentifierDoesNotMatchSOPClass";
+      case STATUS_GET_Error_DataSetDoesNotMatchSOPClass:
+          s = "Error: DataSetDoesNotMatchSOPClass";
           break;
       case STATUS_GET_Cancel_SubOperationsTerminatedDueToCancelIndication:
           s = "Cancel: SubOperationsTerminatedDueToCancelIndication";
@@ -485,7 +462,7 @@ DU_cgetStatusString(Uint16 statusCode)
     }
 
     if (s == NULL) {
-        sprintf(staticBuf, "Unknown Status: 0x%x", (unsigned int)statusCode);
+        OFStandard::snprintf(staticBuf, sizeof(staticBuf), "Unknown Status: 0x%x", (unsigned int)statusCode);
         s = staticBuf;
     }
     return s;
@@ -512,8 +489,8 @@ DU_ncreateStatusString(Uint16 statusCode)
       case STATUS_N_InvalidAttributeValue:
           s = "Failure: InvalidAttributeValue";
           break;
-      case STATUS_N_InvalidObjectInstance:
-          s = "Failure: InvalidObjectInstance";
+      case STATUS_N_InvalidSOPInstance:
+          s = "Failure: InvalidSOPInstance";
           break;
       case STATUS_N_MissingAttribute:
           s = "Failure: MissingAttribute";
@@ -530,8 +507,8 @@ DU_ncreateStatusString(Uint16 statusCode)
       case STATUS_N_NoSuchSOPClass:
           s = "Failure: NoSuchSOPClass";
           break;
-      case STATUS_N_NoSuchObjectInstance:
-          s = "Failure: NoSuchObjectInstance";
+      case STATUS_N_NoSuchSOPInstance:
+          s = "Failure: NoSuchSOPInstance";
           break;
       case STATUS_N_ProcessingFailure:
           s = "Failure: ProcessingFailure";
@@ -556,7 +533,7 @@ DU_ncreateStatusString(Uint16 statusCode)
     }
 
     if (s == NULL) {
-        sprintf(staticBuf, "Unknown Status: 0x%x", (unsigned int)statusCode);
+        OFStandard::snprintf(staticBuf, sizeof(staticBuf), "Unknown Status: 0x%x", (unsigned int)statusCode);
         s = staticBuf;
     }
     return s;
@@ -577,8 +554,8 @@ DU_ngetStatusString(Uint16 statusCode)
       case STATUS_N_DuplicateInvocation:
           s = "Failure: DuplicateInvocation";
           break;
-      case STATUS_N_InvalidObjectInstance:
-          s = "Failure: InvalidObjectInstance";
+      case STATUS_N_InvalidSOPInstance:
+          s = "Failure: InvalidSOPInstance";
           break;
       case STATUS_N_MistypedArgument:
           s = "Failure: MistypedArgument";
@@ -586,8 +563,8 @@ DU_ngetStatusString(Uint16 statusCode)
       case STATUS_N_NoSuchSOPClass:
           s = "Failure: NoSuchSOPClass";
           break;
-      case STATUS_N_NoSuchObjectInstance:
-          s = "Failure: NoSuchObjectInstance";
+      case STATUS_N_NoSuchSOPInstance:
+          s = "Failure: NoSuchSOPInstance";
           break;
       case STATUS_N_ProcessingFailure:
           s = "Failure: ProcessingFailure";
@@ -595,8 +572,8 @@ DU_ngetStatusString(Uint16 statusCode)
       case STATUS_N_ResourceLimitation:
           s = "Failure: ResourceLimitation";
           break;
-      case STATUS_N_AttributeListError:
-          s = "Warning: AttributeListError";
+      case STATUS_N_AttributeListWarning:
+          s = "Warning: AttributeListWarning";
           break;
       case STATUS_N_AttributeValueOutOfRange:
           s = "Warning: AttributeValueOutOfRange";
@@ -612,7 +589,7 @@ DU_ngetStatusString(Uint16 statusCode)
     }
 
     if (s == NULL) {
-          sprintf(staticBuf, "Unknown Status: 0x%x", (unsigned int)statusCode);
+          OFStandard::snprintf(staticBuf, sizeof(staticBuf), "Unknown Status: 0x%x", (unsigned int)statusCode);
           s = staticBuf;
     }
     return s;
@@ -639,8 +616,8 @@ DU_nsetStatusString(Uint16 statusCode)
       case STATUS_N_MistypedArgument:
           s = "Failure: MistypedArgument";
           break;
-      case STATUS_N_InvalidObjectInstance:
-          s = "Failure: InvalidObjectInstance";
+      case STATUS_N_InvalidSOPInstance:
+          s = "Failure: InvalidSOPInstance";
           break;
       case STATUS_N_MissingAttributeValue:
           s = "Failure: MissingAttributeValue";
@@ -651,8 +628,8 @@ DU_nsetStatusString(Uint16 statusCode)
       case STATUS_N_NoSuchSOPClass:
           s = "Failure: NoSuchSOPClass";
           break;
-      case STATUS_N_NoSuchObjectInstance:
-          s = "Failure: NoSuchObjectInstance";
+      case STATUS_N_NoSuchSOPInstance:
+          s = "Failure: NoSuchSOPInstance";
           break;
       case STATUS_N_ProcessingFailure:
           s = "Failure: ProcessingFailure";
@@ -677,7 +654,7 @@ DU_nsetStatusString(Uint16 statusCode)
     }
 
     if (s == NULL) {
-        sprintf(staticBuf, "Unknown Status: 0x%x", (unsigned int)statusCode);
+        OFStandard::snprintf(staticBuf, sizeof(staticBuf), "Unknown Status: 0x%x", (unsigned int)statusCode);
         s = staticBuf;
     }
     return s;
@@ -701,8 +678,8 @@ DU_nactionStatusString(Uint16 statusCode)
       case STATUS_N_InvalidArgumentValue:
           s = "Failure: InvalidArgumentValue";
           break;
-      case STATUS_N_InvalidObjectInstance:
-          s = "Failure: InvalidObjectInstance";
+      case STATUS_N_InvalidSOPInstance:
+          s = "Failure: InvalidSOPInstance";
           break;
       case STATUS_N_MistypedArgument:
           s = "Failure: MistypedArgument";
@@ -716,8 +693,8 @@ DU_nactionStatusString(Uint16 statusCode)
       case STATUS_N_NoSuchSOPClass:
           s = "Failure: NoSuchSOPClass";
           break;
-      case STATUS_N_NoSuchObjectInstance:
-          s = "Failure: NoSuchObjectInstance";
+      case STATUS_N_NoSuchSOPInstance:
+          s = "Failure: NoSuchSOPInstance";
           break;
       case STATUS_N_ProcessingFailure:
           s = "Failure: ProcessingFailure";
@@ -739,7 +716,7 @@ DU_nactionStatusString(Uint16 statusCode)
     }
 
     if (s == NULL) {
-        sprintf(staticBuf, "Unknown Status: 0x%x", (unsigned int)statusCode);
+        OFStandard::snprintf(staticBuf, sizeof(staticBuf), "Unknown Status: 0x%x", (unsigned int)statusCode);
         s = staticBuf;
     }
     return s;
@@ -760,8 +737,8 @@ DU_ndeleteStatusString(Uint16 statusCode)
       case STATUS_N_DuplicateInvocation:
           s = "Failure: DuplicateInvocation";
           break;
-      case STATUS_N_InvalidObjectInstance:
-          s = "Failure: InvalidObjectInstance";
+      case STATUS_N_InvalidSOPInstance:
+          s = "Failure: InvalidSOPInstance";
           break;
       case STATUS_N_MistypedArgument:
           s = "Failure: MistypedArgument";
@@ -769,8 +746,8 @@ DU_ndeleteStatusString(Uint16 statusCode)
       case STATUS_N_NoSuchSOPClass:
           s = "Failure: NoSuchSOPClass";
           break;
-      case STATUS_N_NoSuchObjectInstance:
-          s = "Failure: NoSuchObjectInstance";
+      case STATUS_N_NoSuchSOPInstance:
+          s = "Failure: NoSuchSOPInstance";
           break;
       case STATUS_N_ProcessingFailure:
           s = "Failure: ProcessingFailure";
@@ -792,7 +769,7 @@ DU_ndeleteStatusString(Uint16 statusCode)
     }
 
     if (s == NULL) {
-        sprintf(staticBuf, "Unknown Status: 0x%x", (unsigned int)statusCode);
+        OFStandard::snprintf(staticBuf, sizeof(staticBuf), "Unknown Status: 0x%x", (unsigned int)statusCode);
         s = staticBuf;
     }
     return s;
@@ -816,8 +793,8 @@ DU_neventReportStatusString(Uint16 statusCode)
       case STATUS_N_InvalidArgumentValue:
           s = "Failure: InvalidArgumentValue";
           break;
-      case STATUS_N_InvalidObjectInstance:
-          s = "Failure: InvalidObjectInstance";
+      case STATUS_N_InvalidSOPInstance:
+          s = "Failure: InvalidSOPInstance";
           break;
       case STATUS_N_MistypedArgument:
           s = "Failure: MistypedArgument";
@@ -831,8 +808,8 @@ DU_neventReportStatusString(Uint16 statusCode)
       case STATUS_N_NoSuchSOPClass:
           s = "Failure: NoSuchSOPClass";
           break;
-      case STATUS_N_NoSuchObjectInstance:
-          s = "Failure: NoSuchObjectInstance";
+      case STATUS_N_NoSuchSOPInstance:
+          s = "Failure: NoSuchSOPInstance";
           break;
       case STATUS_N_ProcessingFailure:
           s = "Failure: ProcessingFailure";
@@ -854,7 +831,7 @@ DU_neventReportStatusString(Uint16 statusCode)
     }
 
     if (s == NULL) {
-        sprintf(staticBuf, "Unknown Status: 0x%x", (unsigned int)statusCode);
+        OFStandard::snprintf(staticBuf, sizeof(staticBuf), "Unknown Status: 0x%x", (unsigned int)statusCode);
         s = staticBuf;
     }
     return s;
