@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2016-2019, Open Connections GmbH
+ *  Copyright (C) 2016-2025, Open Connections GmbH
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation are maintained by
@@ -22,6 +22,11 @@
 #include "dcmtk/config/osconfig.h"
 #include "dcmtk/dcmpmap/dpmparametricmapiod.h"
 #include "dcmtk/dcmiod/iodutil.h"
+#include "dcmtk/dcmdata/dcvrobow.h"
+#include "dcmtk/dcmdata/dcvrfl.h"
+#include "dcmtk/dcmdata/dcvrfd.h"
+#include "dcmtk/dcmdata/dcuid.h"
+
 
 // ----------------------------------------------------------------------------
 // Class SetImagePixelModuleVisitor:
@@ -144,12 +149,10 @@ struct DPMParametricMapIOD::ReadVisitor
       {
         for (Uint32 n = 0; n < numFrames; n++)
         {
-          DcmIODTypes::Frame* f = new DcmIODTypes::Frame;
+          DcmIODTypes::Frame<T>* f = new DcmIODTypes::Frame<T>(numBytesFrame);
           if (f)
           {
-            f->length = numBytesFrame;
-            f->pixData = new Uint8[f->length];
-            memcpy(f->pixData, pixData + n*numBytesFrame/2, numBytesFrame);
+            memcpy(f->m_pixData, pixData + n*numBytesFrame/2, numBytesFrame);
             map.m_Frames.push_back(f);
           }
           else
@@ -186,12 +189,10 @@ struct DPMParametricMapIOD::ReadVisitor
       {
         for (Uint32 n=0; n < numFrames; n++)
         {
-          DcmIODTypes::Frame* f = new DcmIODTypes::Frame;
-          if (f)
+          DcmIODTypes::Frame<Float32>* f = new DcmIODTypes::Frame<Float32>(numBytesFrame);
+          if (f && f->m_pixData)
           {
-            f->length = numBytesFrame;
-            f->pixData = new Uint8[f->length];
-            memcpy(f->pixData, pixData + n*numBytesFrame/4, numBytesFrame);
+            memcpy(f->m_pixData, pixData + n*numBytesFrame/4, numBytesFrame);
             map.m_Frames.push_back(f);
           }
           else
@@ -228,12 +229,10 @@ struct DPMParametricMapIOD::ReadVisitor
       {
         for (Uint16 n=0; n < numFrames; n++)
         {
-          DcmIODTypes::Frame* f = new DcmIODTypes::Frame;
-          if (f)
+          DcmIODTypes::Frame<Float64>* f = new DcmIODTypes::Frame<Float64>(numBytesFrame);
+          if (f && f->m_pixData)
           {
-            f->length = numBytesFrame;
-            f->pixData = new Uint8[f->length];
-            memcpy(f->pixData, pixData + n*numBytesFrame/8, numBytesFrame);
+            memcpy(f->m_pixData, pixData + n*numBytesFrame/8, numBytesFrame);
             map.m_Frames.push_back(f);
           }
           else
@@ -304,7 +303,7 @@ struct DPMParametricMapIOD::WriteVisitor
     map.getRows(rows);
     map.getColumns(cols);
     const size_t numFrames = map.m_Frames.size();
-    const size_t numBytesFrame = map.m_Frames[0]->length;
+    const size_t numBytesFrame = map.m_Frames[0]->getLengthInBytes();
     const size_t numPixelsFrame = rows * cols;
     // Creates the correct pixel data element, based on the image pixel module used.
     // I.e. For integer data, the "Pixel Data" element is used, i.e. the DcmElement type
@@ -317,7 +316,7 @@ struct DPMParametricMapIOD::WriteVisitor
     {
       for (size_t f = 0; f < numFrames; ++f)
       {
-        memcpy(ptr, map.m_Frames[f]->pixData, numBytesFrame);
+        memcpy(ptr, map.m_Frames[f]->getPixelData(), numBytesFrame);
         ptr += numPixelsFrame;
       }
       OFCondition result = element.put(item);
@@ -586,12 +585,10 @@ OFCondition DPMParametricMapIOD::Frames<PixelType>::addFrame(PixelType* data,
   {
     if (!perFrameInformation.empty())
     {
-      OFunique_ptr<DcmIODTypes::Frame> f(new DcmIODTypes::Frame);
+      OFunique_ptr<DcmIODTypes::Frame<PixelType> > f(new DcmIODTypes::Frame<PixelType>(numPixels * sizeof(PixelType)));
       if (f)
       {
-        f->length = numPixels * sizeof(PixelType);
-        f->pixData = new Uint8[f->length];
-        memcpy(f->pixData, data, f->length);
+        memcpy(f->m_pixData, data, f->getLengthInBytes());
         m_Map.m_Frames.push_back(f.release());
         OFVector<FGBase*>::const_iterator fg = perFrameInformation.begin();
         while ( result.good() && (fg != perFrameInformation.end()) )
@@ -619,7 +616,11 @@ PixelType* DPMParametricMapIOD::Frames<PixelType>::getFrame(const size_t frameNu
 {
   if (frameNumber < m_Map.m_Frames.size())
   {
-    return (PixelType*)(m_Map.m_Frames[frameNumber]->pixData);
+    DcmIODTypes::Frame<PixelType> * f = OFstatic_cast(DcmIODTypes::Frame<PixelType>*, m_Map.m_Frames[frameNumber]);
+    if (f)
+    {
+      return f->m_pixData;
+    }
   }
   return NULL;
 }
@@ -908,6 +909,34 @@ OFBool DPMParametricMapIOD::check()
 }
 
 
+OFBool DPMParametricMapIOD::getCheckFGOnWrite()
+{
+  return m_FGInterface.getCheckOnWrite();
+}
+
+
+void DPMParametricMapIOD::setCheckFGOnWrite(const OFBool doCheck)
+{
+  m_FGInterface.setCheckOnWrite(doCheck);
+}
+
+
+void DPMParametricMapIOD::setValueCheckOnWrite(const OFBool doCheck)
+{
+  // Forward setting into all modules
+  m_DPMParametricMapSeriesModule.setValueCheckOnWrite(doCheck);
+  m_IODEnhGeneralEquipmentModule.setValueCheckOnWrite(doCheck);
+  m_IODMultiframeDimensionModule.setValueCheckOnWrite(doCheck);
+  m_IODAcquisitionContextModule.setValueCheckOnWrite(doCheck);
+  m_IODCommonInstanceReferenceModule.setValueCheckOnWrite(doCheck);
+
+  m_IODMultiFrameFGModule.setValueCheckOnWrite(doCheck);
+  m_DPMParametricMapImageModule.setValueCheckOnWrite(doCheck);
+  m_DPMParametricMapSeriesModule.setValueCheckOnWrite(doCheck);
+  DcmIODImage::setValueCheckOnWrite(doCheck);
+}
+
+
 OFCondition DPMParametricMapIOD::getColumns(Uint16& cols)
 {
   return getImagePixel().getColumns(cols);
@@ -988,23 +1017,24 @@ OFCondition DPMParametricMapIOD::decompress(DcmDataset& dset)
   // If the original transfer syntax could have been lossy, print warning
   if (dset.hasRepresentation(EXS_LittleEndianExplicit, NULL))
   {
-    if ( xfer.isEncapsulated() && (xfer.getXfer() != EXS_RLELossless) && (xfer.getXfer() != EXS_DeflatedLittleEndianExplicit) )
+    if (xfer.isPixelDataCompressed() && (xfer != EXS_RLELossless))
     {
       DCMPMAP_WARN("Dataset has been compressed using a (possibly) lossy compression scheme (ignored)");
     }
   }
-  // If the original transfer is encapsulated and we do not already have an uncompressed version, decompress or reject the file
-  else if (xfer.isEncapsulated())
+  // If the original transfer syntax refers to compressed pixel data and we do not
+  // already have an uncompressed version, decompress or reject the file
+  else if (xfer.isPixelDataCompressed())
   {
-    // RLE compression is fine (truly lossless). Deflated is handled internally by DCMTK.
-    if (xfer.getXfer() == EXS_RLELossless)
+    // RLE compression is fine (truly lossless)
+    if (xfer == EXS_RLELossless)
     {
       DCMPMAP_DEBUG("DICOM file is RLE-compressed, converting to uncompressed transfer syntax first");
       result = DcmIODUtil::decompress(dset);
     }
     else // We do not accept any transfer syntax that could be lossy compressed
     {
-      DCMPMAP_ERROR("Transfer syntax " << DcmXfer(xfer).getXferName() << " uses lossy compression, not supported for Parametric Map objects!");
+      DCMPMAP_ERROR("Transfer syntax " << xfer.getXferName() << " uses lossy compression, not supported for Parametric Map objects!");
       result = IOD_EC_CannotDecompress;
     }
   }
@@ -1017,6 +1047,25 @@ template class DPMParametricMapIOD::Frames<Sint16>;
 template class DPMParametricMapIOD::Frames<Float32>;
 template class DPMParametricMapIOD::Frames<Float64>;
 
+#ifdef __SUNPRO_CC
+// the SunPro compiler would complain that the DCMTK_DCMPMAP_EXPORT attribute cannot be applied
+// to an explicit template instantiation.
+
+// Helper macro to not to write the same thing multiple times
+#define INSTANTIATE_CREATE(T) template OFvariant<OFCondition,DPMParametricMapIOD>\
+DPMParametricMapIOD::create<T >(const OFString& modality,\
+                                const OFString& seriesNumber,\
+                                const OFString& instanceNumber,\
+                                const Uint16 rows,\
+                                const Uint16 columns,\
+                                const IODEnhGeneralEquipmentModule::EquipmentInfo& equipmentInfo,\
+                                const ContentIdentificationMacro& contentIdentification,\
+                                const OFString& imageFlavor,\
+                                const OFString& derivedPixelContrast,\
+                                const DPMTypes::ContentQualification& contentQualification)
+
+#else
+
 // Helper macro to not to write the same thing multiple times
 #define INSTANTIATE_CREATE(T) template DCMTK_DCMPMAP_EXPORT OFvariant<OFCondition,DPMParametricMapIOD>\
 DPMParametricMapIOD::create<T >(const OFString& modality,\
@@ -1030,6 +1079,8 @@ DPMParametricMapIOD::create<T >(const OFString& modality,\
                                 const OFString& derivedPixelContrast,\
                                 const DPMTypes::ContentQualification& contentQualification)
 
+#endif
+
 // Instantiate all four permitted create() methods
 INSTANTIATE_CREATE(IODImagePixelModule<Uint16>);
 INSTANTIATE_CREATE(IODImagePixelModule<Sint16>);
