@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1998-2012, OFFIS e.V.
+ *  Copyright (C) 1998-2025, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -30,6 +30,7 @@
 #include "dcmtk/dcmdata/dctypes.h"    /* for Uint32 */
 #include "dcmtk/dcmpstat/dvpstyp.h"    /* for DVPS enums */
 #include "dcmtk/dcmpstat/dpdefine.h"
+#include "dcmtk/dcmnet/assoc.h" /* for T_ASC_ProtocolFamily */
 
 class OFConfigFile;
 
@@ -95,6 +96,14 @@ class DCMTK_DCMPSTAT_EXPORT DVConfiguration
    */
   unsigned short getTargetPort(const char *targetID);
 
+  /** returns the PROTOCOL entry for the communication partner with the given
+   *  target ID from the configuration file.
+   *  @param targetID communication target ID, must be one of the target
+   *    identifiers returned by getTargetID().
+   *  @return entry if present and parsable in the config file, ASC_AF_Default otherwise.
+   */
+  T_ASC_ProtocolFamily getTargetProtocol(const char *targetID);
+
   /** returns the TYPE entry for the communication partner with the given
    *  target ID from the configuration file.
    *  @param targetID communication target ID, must be one of the target
@@ -151,6 +160,14 @@ class DCMTK_DCMPSTAT_EXPORT DVConfiguration
    */
   OFBool getTargetBitPreservingMode(const char *targetID);
 
+  /** returns the VALIDATIONMODE entry for the storage peer with the given
+   *  target ID from the configuration file.
+   *  @param targetID communication target ID, must be one of the target
+   *    identifiers returned by getTargetID().
+   *  @return entry if present in the config file, NULL otherwise.
+   */
+  const char *getTargetValidationMode(const char *targetID);
+
   /** returns the CORRECTUIDPADDING entry for the storage peer with the given
    *  target ID from the configuration file.
    *  @param targetID communication target ID, must be one of the target
@@ -191,24 +208,13 @@ class DCMTK_DCMPSTAT_EXPORT DVConfiguration
    */
   const char *getTargetPrivateKeyPassword(const char *targetID);
 
-  /** returns the number of distinct values (separated by backslash characters)
-   *  in the CIPHERSUITES entry for the storage peer with the given
+  /** returns the TLSPROFILE entry for the storage peer with the given
    *  target ID from the configuration file.
    *  @param targetID communication target ID, must be one of the target
    *    identifiers returned by getTargetID().
-   *  @return number of values if entry present in the config file, 0 otherwise.
+   *  @return entry if present in the config file, NULL otherwise.
    */
-  Uint32 getTargetNumberOfCipherSuites(const char *targetID);
-
-  /** returns one value from the CIPHERSUITES entry for the storage peer
-   *  with the given target ID from the configuration file.
-   *  @param targetID communication target ID, must be one of the target
-   *    identifiers returned by getTargetID().
-   *  @param idx index of the value, must be < getTargetNumberOfCipherSuites(targetID)
-   *  @param value the result is both stored in this object and returned as return value.
-   *  @return value if present, NULL otherwise.
-   */
-  const char *getTargetCipherSuite(const char *targetID, Uint32 idx, OFString& value);
+  const char *getTargetTLSProfile(const char *targetID);
 
   /** returns the PEERAUTHENTICATION entry for the communication partner with the given
    *  target ID from the configuration file.
@@ -400,7 +406,7 @@ class DCMTK_DCMPSTAT_EXPORT DVConfiguration
    */
   OFBool getTargetPrinterSupportsAnnotationBoxSOPClass(const char *targetID);
 
-  /** returns OFTrue if an SESSIONLABELANNOTATION entry for the printer 
+  /** returns OFTrue if an SESSIONLABELANNOTATION entry for the printer
    *  with the given target ID from the configuration file exists and is true.
    *  @param targetID communication target ID, must be one of the target
    *    identifiers returned by getTargetID() for peer type DVPSE_printerAny.
@@ -416,7 +422,7 @@ class DCMTK_DCMPSTAT_EXPORT DVConfiguration
    *  @return value if present, NULL otherwise.
    */
   const char *getTargetPrinterAnnotationDisplayFormatID(const char *targetID, OFString& value);
-  
+
   /** returns the first value from the ANNOTATION entry for the printer
    *  with the given target ID from the configuration file.
    *  @param targetID communication target ID, must be one of the target
@@ -424,7 +430,7 @@ class DCMTK_DCMPSTAT_EXPORT DVConfiguration
    *  @return value if present, 0 otherwise.
    */
   Uint16 getTargetPrinterAnnotationPosition(const char *targetID);
-  
+
   /** returns the number of distinct values (separated by backslash characters)
    *  in the FILMSIZEID entry for the printer with the given
    *  target ID from the configuration file.
@@ -606,7 +612,7 @@ class DCMTK_DCMPSTAT_EXPORT DVConfiguration
    */
   OFLogger::LogLevel getLogLevel();
 
-  /** returns the port on which the GUI application accepts notification 
+  /** returns the port on which the GUI application accepts notification
    *  messages from the network processes.
    *  Value is taken from the section GENERAL/APPLICATION/MESSAGEPORT
    *  in the config file.
@@ -666,6 +672,12 @@ class DCMTK_DCMPSTAT_EXPORT DVConfiguration
    */
   unsigned long getQueryRetrieveMaxAssociations();
 
+  /** returns the protocol family to be supported by the Query/Retrieve SCP
+   *  as configured in section GENERAL/QUERY_RETRIEVE/PROTOCOL in the config file.
+   *  @return send application path name or NULL if absent.
+   */
+  T_ASC_ProtocolFamily getQueryRetrieveProtocolFamily();
+
   /** returns the database folder to be used for sending/receiving/browsing.
    *  Value is taken from the section GENERAL/DATABASE/DIRECTORY
    *  in the config file. If absent, a default value is returned.
@@ -947,22 +959,22 @@ class DCMTK_DCMPSTAT_EXPORT DVConfiguration
   double getVOIPresetWindowWidth(const char *modality, Uint32 idx);
 
   /* TLS settings */
-  
-  /** returns the directory in which TLS related files (certificates, keys, 
-   *  random data, Diffie-Hellman parameters etc.) are located. 
+
+  /** returns the directory in which TLS related files (certificates, keys,
+   *  random data, Diffie-Hellman parameters etc.) are located.
    *  @return TLS directory path, NULL if absent.
    */
   const char *getTLSFolder();
 
-  /** returns the directory in which certificates of the trusted 
-   *  Certification Authorities are located. 
+  /** returns the directory in which certificates of the trusted
+   *  Certification Authorities are located.
    *  @return TLS CA Certificate directory path, NULL if absent.
    */
   const char *getTLSCACertificateFolder();
 
   /** returns the file format used for certificates, keys and Diffie-Hellman
    *  parameters. OFTrue for PEM ("privacy enhanced mail") format, OFFalse for
-   *  DER ("distinguished encoding rules") format.  
+   *  DER ("distinguished encoding rules") format.
    *  @return OFTrue for PEM (default), OFFalse for DER.
    */
   OFBool getTLSPEMFormat();
@@ -970,7 +982,7 @@ class DCMTK_DCMPSTAT_EXPORT DVConfiguration
   /* User login settings */
 
   /** returns the directory in which user certificates and keys
-   *  are located. 
+   *  are located.
    *  @return User key/certificate directory path, NULL if absent.
    */
   const char *getUserCertificateFolder();
@@ -994,7 +1006,7 @@ class DCMTK_DCMPSTAT_EXPORT DVConfiguration
    */
   const char *getUserLogin(const char *userID);
 
-  /** returns the human readable name for the given user. 
+  /** returns the human readable name for the given user.
    *  If absent in the config file, returns NULL.
    *  @param userID user ID as returned by getUserID()
    *  @return name for the given user
