@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 1998-2018, OFFIS e.V.
+ *  Copyright (C) 1998-2025, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -26,10 +26,19 @@
 #include "dcmtk/dcmpstat/dvpsdef.h"     /* for constants */
 #include "dcmtk/ofstd/ofstd.h"       /* for class OFStandard */
 
-#define INCLUDE_CSTDIO
-#define INCLUDE_CSTRING
-#define INCLUDE_CCTYPE
-#include "dcmtk/ofstd/ofstdinc.h"
+BEGIN_EXTERN_C
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+END_EXTERN_C
+
+#ifdef HAVE_WINDOWS_H
+#define WIN32_LEAN_AND_MEAN
+#include <winsock2.h>
+#include <windows.h>     /* for GetFileAttributes() */
+#elif defined(HAVE_WINSOCK_H)
+#include <winsock.h>  /* include winsock.h directly i.e. on MacOS */
+#endif /* HAVE_WINDOWS_H */
 
 #ifndef HAVE_WINDOWS_H
 /* some Unix operating systems do not define a prototype for strncasecmp
@@ -62,7 +71,7 @@ extern "C" int strncasecmp(const char *s1, const char *s2, size_t n);
 #define L0_DESCRIPTION                  "DESCRIPTION"
 #define L0_DETAILEDLOG                  "DETAILEDLOG"
 #define L0_DICOMNAME                    "DICOMNAME"
-#define L0_DIFFIEHELLMANPARAMETERS      "DIFFIEHELLMANPARAMETERS"   
+#define L0_DIFFIEHELLMANPARAMETERS      "DIFFIEHELLMANPARAMETERS"
 #define L0_DIRECTORY                    "DIRECTORY"
 #define L0_DISABLENEWVRS                "DISABLENEWVRS"
 #define L0_DISPLAYFORMAT                "DISPLAYFORMAT"
@@ -101,6 +110,7 @@ extern "C" int strncasecmp(const char *s1, const char *s2, size_t n);
 #define L0_PREVIEW                      "PREVIEWSIZE"
 #define L0_PRIVATEKEY                   "PRIVATEKEY"
 #define L0_PRIVATEKEYPASSWORD           "PRIVATEKEYPASSWORD"
+#define L0_PROTOCOL                     "PROTOCOL"
 #define L0_RANDOMSEED                   "RANDOMSEED"
 #define L0_RECEIVER                     "RECEIVER"
 #define L0_RESOLUTION                   "RESOLUTION"
@@ -124,6 +134,7 @@ extern "C" int strncasecmp(const char *s1, const char *s2, size_t n);
 #define L0_USEPEMFORMAT                 "USEPEMFORMAT"
 #define L0_USERKEYDIRECTORY             "USERKEYDIRECTORY"
 #define L0_USETLS                       "USETLS"
+#define L0_VALIDATIONMODE               "VALIDATIONMODE"
 #define L0_WIDTH                        "WIDTH"
 #define L1_APPLICATION                  "APPLICATION"
 #define L1_DATABASE                     "DATABASE"
@@ -181,7 +192,7 @@ static Uint32 countValues(const char *str)
     Uint32 result = 0;
     if (*str) result++;
     char c;
-    while ((c = *str++)) if (c == '\\') result++;
+    while ((c = *str++) != '\0') if (c == '\\') result++;
     return result;
   }
   return 0;
@@ -214,7 +225,7 @@ static int strCompare(const char *str1, const char *str2, size_t len)
   return _strnicmp(str1, str2, len);
 #else
   return strncasecmp(str1, str2, len);
-#endif    
+#endif
 }
 
 
@@ -390,6 +401,19 @@ unsigned short DVConfiguration::getTargetPort(const char *targetID)
   return result;
 }
 
+T_ASC_ProtocolFamily DVConfiguration::getTargetProtocol(const char *targetID)
+{
+  const char *c = getConfigEntry(L2_COMMUNICATION, targetID, L0_PROTOCOL);
+  T_ASC_ProtocolFamily result = ASC_AF_Default;
+  if (c)
+  {
+    if (strCompare(c, "AF_INET6", 8) == 0) result = ASC_AF_INET6;
+    else if (strCompare(c, "AF_INET", 7) == 0) result = ASC_AF_INET;
+    else if (strCompare(c, "AF_UNSPEC", 9) == 0) result = ASC_AF_UNSPEC;
+  }
+  return result;
+}
+
 const char *DVConfiguration::getTargetAETitle(const char *targetID)
 {
   return getConfigEntry(L2_COMMUNICATION, targetID, L0_AETITLE);
@@ -539,6 +563,19 @@ unsigned long DVConfiguration::getQueryRetrieveMaxAssociations()
   return result;
 }
 
+T_ASC_ProtocolFamily DVConfiguration::getQueryRetrieveProtocolFamily()
+{
+  const char *c = getConfigEntry(L2_GENERAL, L1_QUERY_RETRIEVE, L0_PROTOCOL);
+  T_ASC_ProtocolFamily result = ASC_AF_Default;
+  if (c)
+  {
+    if (strCompare(c, "AF_INET6", 8) == 0) result = ASC_AF_INET6;
+    else if (strCompare(c, "AF_INET", 7) == 0) result = ASC_AF_INET;
+    else if (strCompare(c, "AF_UNSPEC", 9) == 0) result = ASC_AF_UNSPEC;
+  }
+  return result;
+}
+
 const char *DVConfiguration::getDatabaseFolder()
 {
   const char *result = getConfigEntry(L2_GENERAL, L1_DATABASE, L0_DIRECTORY);
@@ -886,7 +923,7 @@ Uint32 DVConfiguration::getTargetPrinterNumberOfConfigurationSettings(const char
       char l0_key[80];
       do
       {
-        sprintf(l0_key, "CONFIGURATION_%d", counter++);
+        OFStandard::snprintf(l0_key, sizeof(l0_key), "CONFIGURATION_%d", counter++);
       } while (NULL != pConfig->get_entry(l0_key));
       result = counter - 2;
     }
@@ -897,7 +934,7 @@ Uint32 DVConfiguration::getTargetPrinterNumberOfConfigurationSettings(const char
 const char *DVConfiguration::getTargetPrinterConfigurationSetting(const char *targetID, Uint32 idx)
 {
   char l0_key[80];
-  sprintf(l0_key, "CONFIGURATION_%d", (int)idx+1);
+  OFStandard::snprintf(l0_key, sizeof(l0_key), "CONFIGURATION_%d", (int)idx+1);
   return getConfigEntry(L2_COMMUNICATION, targetID, l0_key);
 }
 
@@ -1280,7 +1317,7 @@ const char *DVConfiguration::getTargetPrinterAnnotationDisplayFormatID(const cha
   copyValue(getConfigEntry(L2_COMMUNICATION, targetID, L0_ANNOTATION), 1, value);
   if (value.length()) return value.c_str(); else return NULL;
 }
-    
+
 Uint16 DVConfiguration::getTargetPrinterAnnotationPosition(const char *targetID)
 {
   OFString value;
@@ -1308,6 +1345,11 @@ OFBool DVConfiguration::getTLSPEMFormat()
   return getConfigBoolEntry(L2_GENERAL, L1_TLS, L0_USEPEMFORMAT, OFTrue);
 }
 
+const char *DVConfiguration::getTargetValidationMode(const char *targetID)
+{
+  return getConfigEntry(L2_COMMUNICATION, targetID, L0_VALIDATIONMODE);
+}
+
 OFBool DVConfiguration::getTargetBitPreservingMode(const char *targetID)
 {
   return getConfigBoolEntry(L2_COMMUNICATION, targetID, L0_BITPRESERVINGMODE, OFFalse);
