@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2015-2019, Open Connections GmbH
+ *  Copyright (C) 2015-2025, Open Connections GmbH
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation are maintained by
@@ -21,10 +21,12 @@
 
 #include "dcmtk/config/osconfig.h"
 
+#include "dcmtk/dcmdata/dcdeftag.h"
 #include "dcmtk/dcmiod/iodutil.h"
 #include "dcmtk/dcmseg/segdoc.h"
 #include "dcmtk/dcmseg/segment.h"
 #include "dcmtk/dcmseg/segtypes.h"
+#include "dcmtk/ofstd/oftypes.h"
 
 OFCondition DcmSegment::create(DcmSegment*& segment,
                                const OFString& segmentLabel,
@@ -66,13 +68,59 @@ OFCondition DcmSegment::create(DcmSegment*& segment,
     return result;
 }
 
+
+
+DcmSegment* DcmSegment::clone(DcmSegmentation* seg)
+{
+    DcmSegment* newSegment = new DcmSegment(*this);
+    if (newSegment != NULL)
+    {
+        if (seg != NULL)
+            newSegment->m_SegmentationDoc = seg;
+        // else: keep reference to same DcmSegmentation document
+    }
+    return newSegment;
+}
+
+DcmSegment::DcmSegment(const DcmSegment& rhs)
+    : m_SegmentationDoc(rhs.m_SegmentationDoc)
+    , m_SegmentNumber(rhs.m_SegmentNumber)
+    , m_SegmentDescription(rhs.m_SegmentDescription)
+    , m_SegmentAlgorithmName(rhs.m_SegmentAlgorithmName)
+    , m_SegmentationAlgorithmIdentification(rhs.m_SegmentationAlgorithmIdentification)
+    , m_RecommendedDisplayGrayscaleValue(rhs.m_RecommendedDisplayGrayscaleValue)
+    , m_RecommendedDisplayCIELabValue(rhs.m_RecommendedDisplayCIELabValue)
+    , m_TrackingID(rhs.m_TrackingID)
+    , m_TrackingUID(rhs.m_TrackingUID)
+    , m_Rules(rhs.m_Rules->clone())
+{
+}
+
+DcmSegment& DcmSegment::operator=(const DcmSegment& rhs)
+{
+    if (this != &rhs)
+    {
+        m_SegmentationDoc                        = rhs.m_SegmentationDoc;
+        m_SegmentNumber                          = rhs.m_SegmentNumber;
+        m_SegmentDescription                     = rhs.m_SegmentDescription;
+        m_SegmentAlgorithmName                   = rhs.m_SegmentAlgorithmName;
+        m_SegmentationAlgorithmIdentification    = rhs.m_SegmentationAlgorithmIdentification;
+        m_RecommendedDisplayGrayscaleValue       = rhs.m_RecommendedDisplayGrayscaleValue;
+        m_RecommendedDisplayCIELabValue          = rhs.m_RecommendedDisplayCIELabValue;
+        m_TrackingID                             = rhs.m_TrackingID;
+        m_TrackingUID                            = rhs.m_TrackingUID;
+        m_Rules.reset(rhs.m_Rules->clone());
+    }
+    return *this;
+}
+
 OFCondition DcmSegment::read(DcmItem& item, const OFBool clearOldData)
 {
     if (clearOldData)
         clearData();
 
     m_SegmentDescription.read(item);
-    DcmIODUtil::getAndCheckElementFromDataset(item, m_SegmentAlgorithmName, m_Rules.getByTag(DCM_SegmentAlgorithmName));
+    DcmIODUtil::getAndCheckElementFromDataset(item, m_SegmentAlgorithmName, m_Rules->getByTag(DCM_SegmentAlgorithmName));
 
     DcmIODUtil::readSingleItem<AlgorithmIdentificationMacro>(item,
                                                              DCM_SegmentationAlgorithmIdentificationSequence,
@@ -81,11 +129,12 @@ OFCondition DcmSegment::read(DcmItem& item, const OFBool clearOldData)
                                                              "Segmentation Image Module");
 
     DcmIODUtil::getAndCheckElementFromDataset(
-        item, m_RecommendedDisplayGrayscaleValue, m_Rules.getByTag(DCM_RecommendedDisplayGrayscaleValue));
+        item, m_RecommendedDisplayGrayscaleValue, m_Rules->getByTag(DCM_RecommendedDisplayGrayscaleValue));
     DcmIODUtil::getAndCheckElementFromDataset(
-        item, m_RecommendedDisplayCIELabValue, m_Rules.getByTag(DCM_RecommendedDisplayCIELabValue));
-    DcmIODUtil::getAndCheckElementFromDataset(item, m_TrackingID, m_Rules.getByTag(DCM_TrackingID));
-    DcmIODUtil::getAndCheckElementFromDataset(item, m_TrackingUID, m_Rules.getByTag(DCM_TrackingUID));
+        item, m_RecommendedDisplayCIELabValue, m_Rules->getByTag(DCM_RecommendedDisplayCIELabValue));
+    DcmIODUtil::getAndCheckElementFromDataset(item, m_TrackingID, m_Rules->getByTag(DCM_TrackingID));
+    DcmIODUtil::getAndCheckElementFromDataset(item, m_TrackingUID, m_Rules->getByTag(DCM_TrackingUID));
+    DcmIODUtil::getAndCheckElementFromDataset(item, m_SegmentNumber, m_Rules->getByTag(DCM_SegmentNumber));
 
     return EC_Normal;
 }
@@ -94,7 +143,7 @@ OFCondition DcmSegment::write(DcmItem& item)
 {
     OFCondition result;
     result = m_SegmentDescription.write(item);
-    DcmIODUtil::copyElementToDataset(result, item, m_SegmentAlgorithmName, m_Rules.getByTag(DCM_SegmentAlgorithmName));
+    DcmIODUtil::copyElementToDataset(result, item, m_SegmentAlgorithmName, m_Rules->getByTag(DCM_SegmentAlgorithmName));
 
     if (result.good() && m_SegmentationAlgorithmIdentification.check(OFTrue /* quiet */).good())
     {
@@ -107,17 +156,20 @@ OFCondition DcmSegment::write(DcmItem& item)
     }
 
     DcmIODUtil::copyElementToDataset(
-        result, item, m_RecommendedDisplayGrayscaleValue, m_Rules.getByTag(DCM_RecommendedDisplayGrayscaleValue));
+        result, item, m_RecommendedDisplayGrayscaleValue, m_Rules->getByTag(DCM_RecommendedDisplayGrayscaleValue));
     DcmIODUtil::copyElementToDataset(
-        result, item, m_RecommendedDisplayCIELabValue, m_Rules.getByTag(DCM_RecommendedDisplayCIELabValue));
-    DcmIODUtil::copyElementToDataset(result, item, m_TrackingID, m_Rules.getByTag(DCM_TrackingID));
-    DcmIODUtil::copyElementToDataset(result, item, m_TrackingUID, m_Rules.getByTag(DCM_TrackingUID));
+        result, item, m_RecommendedDisplayCIELabValue, m_Rules->getByTag(DCM_RecommendedDisplayCIELabValue));
+    DcmIODUtil::copyElementToDataset(result, item, m_TrackingID, m_Rules->getByTag(DCM_TrackingID));
+    DcmIODUtil::copyElementToDataset(result, item, m_TrackingUID, m_Rules->getByTag(DCM_TrackingUID));
+    m_SegmentNumber.putUint16(getSegmentNumber());
+    DcmIODUtil::copyElementToDataset(result, item, m_SegmentNumber, m_Rules->getByTag(DCM_SegmentNumber));
 
     return result;
 }
 
 void DcmSegment::clearData()
 {
+    m_SegmentNumber.clear();
     m_SegmentDescription.clearData();
     m_SegmentAlgorithmName.clear();
     m_SegmentationAlgorithmIdentification.clearData();
@@ -133,8 +185,10 @@ DcmSegment::~DcmSegment()
 }
 
 // protected default constructor
+
 DcmSegment::DcmSegment()
     : m_SegmentationDoc(NULL)
+    , m_SegmentNumber(DCM_SegmentNumber)
     , m_SegmentDescription()
     , m_SegmentAlgorithmName(DCM_SegmentAlgorithmName)
     , m_SegmentationAlgorithmIdentification()
@@ -142,33 +196,44 @@ DcmSegment::DcmSegment()
     , m_RecommendedDisplayCIELabValue(DCM_RecommendedDisplayCIELabValue)
     , m_TrackingID(DCM_TrackingID)
     , m_TrackingUID(DCM_TrackingUID)
-    , m_Rules()
+    , m_Rules(new IODRules())
 {
     initIODRules();
 }
 
 void DcmSegment::initIODRules()
 {
-    m_Rules.addRule(new IODRule(DCM_SegmentAlgorithmName, "1", "1C", "SegmentationImageModule", DcmIODTypes::IE_IMAGE),
+    m_Rules->addRule(new IODRule(DCM_SegmentNumber, "1", "1", "SegmentationImageModule", DcmIODTypes::IE_IMAGE),
                     OFTrue);
-    m_Rules.addRule(
+    m_Rules->addRule(new IODRule(DCM_SegmentAlgorithmName, "1", "1C", "SegmentationImageModule", DcmIODTypes::IE_IMAGE),
+                    OFTrue);
+    m_Rules->addRule(
         new IODRule(DCM_RecommendedDisplayGrayscaleValue, "1", "3", "SegmentationImageModule", DcmIODTypes::IE_IMAGE),
         OFTrue);
-    m_Rules.addRule(
+    m_Rules->addRule(
         new IODRule(DCM_RecommendedDisplayCIELabValue, "3", "3", "SegmentationImageModule", DcmIODTypes::IE_IMAGE),
         OFTrue);
-    m_Rules.addRule(new IODRule(DCM_TrackingID, "1", "1C", "SegmentationImageModule", DcmIODTypes::IE_IMAGE), OFTrue);
-    m_Rules.addRule(new IODRule(DCM_TrackingUID, "1", "1C", "SegmentationImageModule", DcmIODTypes::IE_IMAGE), OFTrue);
+    m_Rules->addRule(new IODRule(DCM_TrackingID, "1", "1C", "SegmentationImageModule", DcmIODTypes::IE_IMAGE), OFTrue);
+    m_Rules->addRule(new IODRule(DCM_TrackingUID, "1", "1C", "SegmentationImageModule", DcmIODTypes::IE_IMAGE), OFTrue);
 }
 
 // -------------- getters --------------------
 
-unsigned int DcmSegment::getSegmentNumber()
+Uint16 DcmSegment::getSegmentNumber()
 {
-    unsigned int value = 0;
+    Uint16 value = 0;
     if (m_SegmentationDoc != NULL)
     {
-        m_SegmentationDoc->getSegmentNumber(this, value);
+        size_t big = 0;
+        m_SegmentationDoc->getSegmentNumber(this, big);
+        if (big <= DCM_SEG_MAX_SEGMENTS)
+        {
+            value = OFstatic_cast(Uint16, big);
+        }
+        else
+        {
+            DCMSEG_ERROR("More segments (" << big << ") defined than permitted (" << DCM_SEG_MAX_SEGMENTS << ")");
+        }
     }
     return value;
 }
@@ -245,6 +310,25 @@ OFCondition DcmSegment::getTrackingUID(OFString& value, const signed long pos)
     return DcmIODUtil::getStringValueFromElement(m_TrackingUID, value, pos);
 }
 
+Uint16 DcmSegment::getSegmentNumberRead()
+{
+    OFVector<Uint16> values;
+    DcmIODUtil::getUint16ValuesFromElement(m_SegmentNumber, values);
+    if (values.size() >0 )
+    {
+        return values[0];
+    }
+    else
+    {
+        return 0;
+    }
+}
+
+OFshared_ptr<IODRules> DcmSegment::getIODRules()
+{
+    return m_Rules;
+}
+
 // -------------- setters --------------------
 
 OFCondition DcmSegment::setSegmentLabel(const OFString& value, const OFBool checkValue)
