@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2015-2024, OFFIS e.V.
+ *  Copyright (C) 2015-2025, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -30,5 +30,8 @@ OFTEST_REGISTER(dcmseg_packAndUnpackBinaryFrame);
 OFTEST_REGISTER(dcmseg_concatBinaryFrames);
 OFTEST_REGISTER(dcmseg_roundtrip);
 OFTEST_REGISTER(dcmseg_concat_binary);
-
+OFTEST_REGISTER(dcmseg_labelmap_8bit_mono2);
+OFTEST_REGISTER(dcmseg_labelmap_16bit_mono2);
+OFTEST_REGISTER(dcmseg_labelmap_8bit_palette);
+OFTEST_REGISTER(dcmseg_labelmap_16bit_palette);
 OFTEST_MAIN("dcmseg")
