@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2000-2017, OFFIS e.V.
+ *  Copyright (C) 2000-2024, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -47,32 +47,6 @@ DSRNumTreeNode::~DSRNumTreeNode()
 }
 
 
-OFBool DSRNumTreeNode::operator==(const DSRDocumentTreeNode &node) const
-{
-    /* call comparison operator of base class (includes check of value type) */
-    OFBool result = DSRDocumentTreeNode::operator==(node);
-    if (result)
-    {
-        /* it's safe to cast the type since the value type has already been checked */
-        result = DSRNumericMeasurementValue::operator==(*OFstatic_cast(const DSRNumericMeasurementValue *, OFstatic_cast(const DSRNumTreeNode *, &node)));
-    }
-    return result;
-}
-
-
-OFBool DSRNumTreeNode::operator!=(const DSRDocumentTreeNode &node) const
-{
-    /* call comparison operator of base class (includes check of value type) */
-    OFBool result = DSRDocumentTreeNode::operator!=(node);
-    if (!result)
-    {
-        /* it's safe to cast the type since the value type has already been checked */
-        result = DSRNumericMeasurementValue::operator!=(*OFstatic_cast(const DSRNumericMeasurementValue *, OFstatic_cast(const DSRNumTreeNode *, &node)));
-    }
-    return result;
-}
-
-
 DSRNumTreeNode *DSRNumTreeNode::clone() const
 {
     return new DSRNumTreeNode(*this);
@@ -86,6 +60,32 @@ void DSRNumTreeNode::clear()
 }
 
 
+OFBool DSRNumTreeNode::isEqual(const DSRDocumentTreeNode &node) const
+{
+    /* call comparison operator of base class (includes check of value type) */
+    OFBool result = DSRDocumentTreeNode::isEqual(node);
+    if (result)
+    {
+        /* it's safe to cast the type since the value type has already been checked */
+        result = DSRNumericMeasurementValue::isEqual(OFstatic_cast(const DSRNumTreeNode &, node).getValue());
+    }
+    return result;
+}
+
+
+OFBool DSRNumTreeNode::isNotEqual(const DSRDocumentTreeNode &node) const
+{
+    /* call comparison operator of base class (includes check of value type) */
+    OFBool result = DSRDocumentTreeNode::isNotEqual(node);
+    if (!result)
+    {
+        /* it's safe to cast the type since the value type has already been checked */
+        result = DSRNumericMeasurementValue::isNotEqual(OFstatic_cast(const DSRNumTreeNode &, node).getValue());
+    }
+    return result;
+}
+
+
 OFBool DSRNumTreeNode::isValid() const
 {
     /* ConceptNameCodeSequence required */
@@ -163,3 +163,19 @@ OFCondition DSRNumTreeNode::renderHTMLContentItem(STD_NAMESPACE ostream &docStre
     }
     return result;
 }
+
+
+// comparison operators
+
+OFBool operator==(const DSRNumTreeNode &lhs,
+                  const DSRNumTreeNode &rhs)
+{
+    return lhs.isEqual(rhs);
+}
+
+
+OFBool operator!=(const DSRNumTreeNode &lhs,
+                  const DSRNumTreeNode &rhs)
+{
+    return lhs.isNotEqual(rhs);
+}
