@@ -187,7 +187,7 @@ OFTEST(oficonv_open)
 
 OFTEST(oficonv_canonicalize)
 {
-  const char *c;
+  char *c;
 
   // these are the canonical names we expect as result
   OFString latin1("ISO-8859-1");
@@ -213,46 +213,46 @@ OFTEST(oficonv_canonicalize)
 
   // exercise all synonyms for ISO-8859-1 as defined in oficonv/datasrc/esdb/esdb.alias,
   // with mixed uppercase/lowercase spellings
-  c = OFiconv_canonicalize("cp819");            OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("csisolatin1");      OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("ibm819");           OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("iso_8859-1");       OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("iso_8859-1:1987");  OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("iso-ir-100");       OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("iso8859-1");        OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("l1");               OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("latin1");           OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("CP819");            OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("CSISOLATIN1");      OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("IBM819");           OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("ISO_8859-1");       OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("ISO_8859-1:1987");  OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("ISO-IR-100");       OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("ISO8859-1");        OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("L1");               OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("LATIN1");           OFCHECK(c && latin1 == c);
-  c = OFiconv_canonicalize("cSiSoLaTiN1");      OFCHECK(c && latin1 == c);
+  c = OFiconv_canonicalize("cp819");            OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("csisolatin1");      OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("ibm819");           OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("iso_8859-1");       OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("iso_8859-1:1987");  OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("iso-ir-100");       OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("iso8859-1");        OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("l1");               OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("latin1");           OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("CP819");            OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("CSISOLATIN1");      OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("IBM819");           OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("ISO_8859-1");       OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("ISO_8859-1:1987");  OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("ISO-IR-100");       OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("ISO8859-1");        OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("L1");               OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("LATIN1");           OFCHECK(c && latin1 == c);  if (c) free(c);
+  c = OFiconv_canonicalize("cSiSoLaTiN1");      OFCHECK(c && latin1 == c);  if (c) free(c);
 
   // exercise at least one synonym for each supported character set
-  c = OFiconv_canonicalize("gb2312");           OFCHECK(c && euc_cn == c);
-  c = OFiconv_canonicalize("iso-ir-149");       OFCHECK(c && euc_kr == c);
-  c = OFiconv_canonicalize("iso-ir-101");       OFCHECK(c && latin2 == c);
-  c = OFiconv_canonicalize("latin3");           OFCHECK(c && latin3 == c);
-  c = OFiconv_canonicalize("iso_8859-4");       OFCHECK(c && latin4 == c);
-  c = OFiconv_canonicalize("cyrillic");         OFCHECK(c && cyrillic == c);
-  c = OFiconv_canonicalize("arabic");           OFCHECK(c && arabic == c);
-  c = OFiconv_canonicalize("greek");            OFCHECK(c && greek == c);
-  c = OFiconv_canonicalize("hebrew");           OFCHECK(c && hebrew == c);
-  c = OFiconv_canonicalize("iso-ir-148");       OFCHECK(c && latin5 == c);
-  c = OFiconv_canonicalize("tis-620");          OFCHECK(c && thai == c);
-  c = OFiconv_canonicalize("iso-ir-203");       OFCHECK(c && latin9 == c);
-  c = OFiconv_canonicalize("ascii");            OFCHECK(c && ascii == c);
-  c = OFiconv_canonicalize("jis_x0201");        OFCHECK(c && jisx0201 == c);
-  c = OFiconv_canonicalize("jis0208");          OFCHECK(c && jisx0208 == c);
-  c = OFiconv_canonicalize("ms_kanji");         OFCHECK(c && shift_jis == c);
-  c = OFiconv_canonicalize("jis_x0212");        OFCHECK(c && jisx0212 == c);
-  c = OFiconv_canonicalize("utf8");             OFCHECK(c && utf8 == c);
-  c = OFiconv_canonicalize("unicode");          OFCHECK(c && utf16 == c);
+  c = OFiconv_canonicalize("gb2312");           OFCHECK(c && euc_cn == c);     if (c) free(c);
+  c = OFiconv_canonicalize("iso-ir-149");       OFCHECK(c && euc_kr == c);     if (c) free(c);
+  c = OFiconv_canonicalize("iso-ir-101");       OFCHECK(c && latin2 == c);     if (c) free(c);
+  c = OFiconv_canonicalize("latin3");           OFCHECK(c && latin3 == c);     if (c) free(c);
+  c = OFiconv_canonicalize("iso_8859-4");       OFCHECK(c && latin4 == c);     if (c) free(c);
+  c = OFiconv_canonicalize("cyrillic");         OFCHECK(c && cyrillic == c);   if (c) free(c);
+  c = OFiconv_canonicalize("arabic");           OFCHECK(c && arabic == c);     if (c) free(c);
+  c = OFiconv_canonicalize("greek");            OFCHECK(c && greek == c);      if (c) free(c);
+  c = OFiconv_canonicalize("hebrew");           OFCHECK(c && hebrew == c);     if (c) free(c);
+  c = OFiconv_canonicalize("iso-ir-148");       OFCHECK(c && latin5 == c);     if (c) free(c);
+  c = OFiconv_canonicalize("tis-620");          OFCHECK(c && thai == c);       if (c) free(c);
+  c = OFiconv_canonicalize("iso-ir-203");       OFCHECK(c && latin9 == c);     if (c) free(c);
+  c = OFiconv_canonicalize("ascii");            OFCHECK(c && ascii == c);      if (c) free(c);
+  c = OFiconv_canonicalize("jis_x0201");        OFCHECK(c && jisx0201 == c);   if (c) free(c);
+  c = OFiconv_canonicalize("jis0208");          OFCHECK(c && jisx0208 == c);   if (c) free(c);
+  c = OFiconv_canonicalize("ms_kanji");         OFCHECK(c && shift_jis == c);  if (c) free(c);
+  c = OFiconv_canonicalize("jis_x0212");        OFCHECK(c && jisx0212 == c);   if (c) free(c);
+  c = OFiconv_canonicalize("utf8");             OFCHECK(c && utf8 == c);       if (c) free(c);
+  c = OFiconv_canonicalize("unicode");          OFCHECK(c && utf16 == c);      if (c) free(c);
 }
 
 
@@ -648,3 +648,8 @@ OFTEST(oficonvctl)
     OFiconv_close(id3);
   }
 }
+
+OFTEST(oficonv_cleanup)
+{
+    OFiconv_cleanup();
+}
