@@ -1,6 +1,6 @@
 /*
  *
- *  Copyright (C) 2001-2014, OFFIS e.V.
+ *  Copyright (C) 2001-2025, OFFIS e.V.
  *  All rights reserved.  See COPYRIGHT file for details.
  *
  *  This software and supporting documentation were developed by
@@ -28,11 +28,9 @@
 #include "dcmtk/ofstd/oftypes.h"    /* for class OFBool */
 #include "dcmtk/ofstd/ofstring.h"   /* for class OFString */
 #include "dcmtk/ofstd/ofcast.h"
-
-#define INCLUDE_CSTRING             /* for strdup() */
-#define INCLUDE_CSTDLIB             /* for free() */
-#include "dcmtk/ofstd/ofstdinc.h"
-
+#include "dcmtk/ofstd/ofdiag.h"
+#include <cstring>
+#include <cstdlib>                  /* for free() */
 
 // include this file in doxygen documentation
 
@@ -103,7 +101,8 @@ struct DCMTK_OFSTD_EXPORT OFConditionConst
  *  All constants defined here use module number 0, which is reserved for
  *  global definitions. Other constants are defined in other modules.
  */
-//@{
+///@{
+
 /// condition constant: successful completion
 extern DCMTK_OFSTD_EXPORT const OFConditionConst EC_Normal;
 /// condition constant: error, function called with illegal parameters
@@ -132,6 +131,9 @@ extern DCMTK_OFSTD_EXPORT const OFConditionConst EC_CouldNotGenerateDirectoryNam
 /// condition constant: error, call to setuid() failed
 extern DCMTK_OFSTD_EXPORT const OFConditionConst EC_setuidFailed;
 
+/// condition constant: error, function not yet implemented
+extern DCMTK_OFSTD_EXPORT const OFConditionConst EC_NotYetImplemented;
+
 /// status code constant: error, cannot open character encoding
 extern DCMTK_OFSTD_EXPORT const unsigned short EC_CODE_CannotOpenEncoding;
 /// status code constant: error, cannot close character encoding
@@ -140,10 +142,21 @@ extern DCMTK_OFSTD_EXPORT const unsigned short EC_CODE_CannotCloseEncoding;
 extern DCMTK_OFSTD_EXPORT const unsigned short EC_CODE_CannotConvertEncoding;
 /// status code constant: error, cannot control character encoding converter
 extern DCMTK_OFSTD_EXPORT const unsigned short EC_CODE_CannotControlConverter;
-
 /// status code constant: error, cannot create directory
 extern DCMTK_OFSTD_EXPORT const unsigned short EC_CODE_CannotCreateDirectory;
-//@}
+
+/// condition constant: error, IPC message queue already initialized
+extern DCMTK_OFSTD_EXPORT const OFConditionConst EC_IPCMessageQueueExists;
+/// condition constant: error, IPC message queue not initialized
+extern DCMTK_OFSTD_EXPORT const OFConditionConst EC_IPCMessageNoQueue;
+/// condition constant: error, IPC message queue operation failed
+extern DCMTK_OFSTD_EXPORT const OFConditionConst EC_IPCMessageQueueFailure;
+/// condition constant: error, IPC message queue empty
+extern DCMTK_OFSTD_EXPORT const OFConditionConst EC_IPCMessageQueueEmpty;
+/// condition constant: error, IPC message empty (zero length)
+extern DCMTK_OFSTD_EXPORT const OFConditionConst EC_IPCEmptyMessage;
+
+///@}
 
 
 /** use this macro for creating static OFCondition instances. Instead of an
@@ -211,6 +224,11 @@ public:
     }
   }
 
+// suppress spurious gcc 12 warning about use-after-free.
+// this is a known bug in gcc.
+#include DCMTK_DIAGNOSTIC_PUSH
+#include DCMTK_DIAGNOSTIC_IGNORE_USE_AFTER_FREE
+
   /// destructor
   ~OFCondition()
   {
@@ -220,6 +238,8 @@ public:
     }
   }
 
+#include DCMTK_DIAGNOSTIC_POP
+
   /** copy assignment operator
    *  @param arg The OFCondition instance to copy
    *  @return *this
@@ -309,6 +329,9 @@ public:
   /* Implicit conversion from OFCondition to bool might
    * not always be a good idea since it can hide unwanted constructs.
    * Therefore, we disable this operator by default.
+   * @remark this method is only available if DCMTK is compiled with
+   * implicit conversion from OFCondition to bool being enabled (see macro
+   * OFCONDITION_IMPLICIT_BOOL_CONVERSION)
    */
 
   /** conversion operator to bool.
